est_lm_cov <- function(S,X1,X2,yv=rep(1,ns),k,start=0,tol=10^-6,param="multilogit",
                       Psi=NULL,Be,Ga,fort=F,output=FALSE,info=FALSE){

# X = array of time-varying covariates organized as S
# these covariates affect initial and transition probabilities
# be = parameters on the initial probabilities
# start = initialization (0 = deterministic, 1 = random, 2 = initial values in input)
#         if start=0 or start=1, but an argument is given in input, then it is not updated anymore
# param = type of parametrization for the transition probabilities:
#         multilogit = standard multinomial logit for every row of the transition matrix
#         difflogit  = multinomial logit based on the difference between two sets of parameters
# for   = fortran use (FALSE for not use fortran)
# X1    = design matrix for the initial probabilities (n by n.cov.)
# X2    = design matrix for the initial probabilities (n by TT-1 by n.cov.)
# info  = TRUE for computing the information and standard errors

# Preliminaries
    check_der = FALSE # to check score and info
    if(fort!=FALSE) fort = TRUE
   	sS = dim(S)
  	ns = sS[1]
  	TT = sS[2]
	n = sum(yv)
  	if(length(sS)==2) r = 1
  	else r = sS[3]
	Sv = matrix(S,ns*TT,r)
	if(r==1){
		b = max(S); mb = b; sb = b
	    Co = cbind(-diag(b),diag(b))
	 	Ma = cbind(lower.tri(matrix(1,b,b), diag = TRUE),rep(0,b))
  		Ma = rbind(Ma,1-Ma)
  	}else{
		b = rep(0,r)
		for(j in 1:r) b[j] = max(S[,,j])
		mb = max(b); sb = sum(b)
		Matr = vector("list",r)
		for(j in 1:r){
			Matr[[j]]$Co = cbind(-diag(b[j]),diag(b[j]))
		 	Ma = cbind(lower.tri(matrix(1,b[j],b[j]), diag = TRUE),rep(0,b[j]))
  			Matr[[j]]$Ma = rbind(Ma,1-Ma)
		}
	}
	th = NULL; sc = NULL
	J = NULL
	  		
# Covariate structure and related matrices: initial probabilities
	nc1 = dim(X1)[2] # number of covariates on the initial probabilities
	if(k == 2) GBe = as.matrix(c(0,1)) else{
		GBe = diag(k); GBe = GBe[,-1]
	}
	out = aggr_data(X1)
	Xdis = out$data_dis
	Xlab = out$label
	Xndis = max(Xlab)
	XXdis = array(0,c(k,(k-1)*(nc1+1),Xndis))
	for(i in 1:Xndis){
		xdis = c(1,Xdis[i,])
		XXdis[,,i] = GBe%*%(diag(k-1)%x%t(xdis))
	}

# for the transition probabilities
    nc2 = dim(X2)[3] # number of covariates on the transition probabilities
	Z = NULL
	for(t in 1:(TT-1)) Z = rbind(Z,X2[,t,])
	out = aggr_data(Z); Zdis = out$data_dis; Zlab = out$label; Zndis = max(Zlab)
	if(param=="multilogit"){
    	ZZdis = array(0,c(k,(k-1)*(nc2+1),Zndis,k))
	    for(h in 1:k){
		    if(k==2){
			    if(h == 1) GGa = as.matrix(c(0,1)) else GGa = as.matrix(c(1,0))
		    }else{
			    GGa = diag(k); GGa = GGa[,-h]
		    }  		
		    for(i in 1:Zndis){
			    zdis = c(1,Zdis[i,])
			    ZZdis[,,i,h] = GGa%*%(diag(k-1)%x%t(zdis))
		    }
	    }
	 }else if(param=="difflogit"){
        Zlab = (((Zlab-1)*k)%x%rep(1,k))+rep(1,ns*(TT-1))%x%(1:k)
        ZZdis = array(0,c(k,k*(k-1)+(k-1)*nc2,Zndis*k))
        j = 0
		for(i in 1:Zndis){
            for(h in 1:k){
                j = j+1
                if(k==2){
			      if(h == 1) GGa = as.matrix(c(0,1)) else GGa = as.matrix(c(1,0))
		        }else{
			        GGa = diag(k); GGa = GGa[,-h]
		        }  		
			    u = matrix(0,1,k); u[1,h] = 1
			    U = diag(k); U[,h] = U[,h]-1
			    U = U[,-1]
		        ZZdis[,,j] = cbind(u%x%GGa,U%x%t(Zdis[i,]))            
            }
	    }
    }
	
# for information matrix
  	if(info){
  	    Am = vector("list",r)
  	    for(j in 1:r) Am[[j]] = rbind(rep(0,b[j]),diag(b[j]))
  	}
# When there is just 1 latent class
  	if(k == 1){
		Piv = rep(1,n); Pi = 1
   	 	P = matrix(0,mb+1,r)
	    	for(t in 1:TT){
	      	for(j in 1:r){
		        	for(y in 0:b[j]){
		        		if(r==1) ind = which(S[,t]) else ind = which(S[,t,j]==y)
		  		        P[y+1,j] = P[y+1,j]+sum(yv[ind])
				}
	   		}
	    }
	    Psi = P/(n*TT)
	    pm = rep(1,ns)
	    for(t in 1:TT) for(j in 1:r) pm = pm*Psi[S[,t,j]+1,j]
	    lk = sum(yv*log(pm))
		if(r==1) np = k*mb*r else np = k*sum(b)
	    aic = -2*lk+np*2
	    bic = -2*lk+np*log(n)
#    	out = list(lk=lk,piv=piv,Pi=Pi,Psi=Psi,np=np,aic=aic,bic=bic,lkv=NULL,J=NULL,V=NULL,th=NULL,sc=NULL)
		out = list(lk=lk,piv=piv,Pi=Pi,Psi=Psi,np=np,aic=aic,bic=bic,lkv=NULL,V=NULL)
	    return(out)
  	}
    time = proc.time()

# Starting values: deterministic initialization
	if(start == 0){
		if(is.null(Psi)){
			upPsi = T
            P = matrix(0,mb+1,r)
            	for(t in 1:TT){
            		for(j in 1:r){
           	    		for(y in 0:b[j]){
	           	    		if(r==1) ind = which(S[,t]==y) else ind = which(S[,t,j]==y)
        			    		P[y+1,j] = P[y+1,j]+sum(yv[ind])
           			    }
           		    }
       		    }
       	        if(r==1) E = Co%*%log(Ma%*%P) else{
       		        E = matrix(0,mb,r)
    			    for(j in 1:r){
    				    Co = Matr[[j]]$Co; Ma = Matr[[j]]$Ma
    				    E[1:b[j],j] = Co%*%log(Ma%*%P[1:(b[j]+1),j])
			        }   		
       	        }
	  	        Psi = array(0,c(mb+1,k,r)); Eta = array(0,c(mb,k,r))
        	    grid = seq(-k,k,2*k/(k-1))
        	    for(c in 1:k){
         		    for(j in 1:r){
	     			    etac = E[1:b[j],j]+grid[c]
          			    Eta[1:b[j],c,j] = etac
          			    Psi[1:(b[j]+1),c,j] = invglob(etac)
          		    }
       	        }
       	    }else upPsi = F
# parameters on initial probabilities
       	be = array(0,(nc1+1)*(k-1))
       	out = prob_multilogit(XXdis,be,Xlab,fort)
       	Piv = out$P; Pivdis = out$Pdis
# parameters on transition probabilities
        if(param=="multilogit"){
            Ga = matrix(0,(nc2+1)*(k-1),k)
            Ga[1+(0:(k-2))*(nc2+1),] = -log(10)
			PIdis = array(0,c(Zndis,k,k)); PI = array(0,c(k,k,ns,TT))
			for(h in 1:k){
			    out = prob_multilogit(ZZdis[,,,h],Ga[,h],Zlab,fort)
			    PIdis[,,h] = out$Pdis; PI[h,,,2:TT] = array(as.vector(t(out$P)),c(1,k,ns,TT-1))
		    }
		}else if(param=="difflogit"){
             Ga = matrix(0,k*(k-1)+(k-1)*nc2)
             Ga[1:((h-1)*k)] = -log(10)
             PI = array(0,c(k,k,ns,TT))
             out = prob_multilogit(ZZdis,Ga,Zlab,fort)
   		    PIdis = out$Pdis; PI[,,,2:TT] = array(as.vector(t(out$P)),c(1,k,ns,TT-1))
		}
  	}
# random initialization
  	if(start==1){
  		if(is.null(Psi)){
  			upPsi = T
    	    Psi = array(0,c(mb+1,k,r)) 
	        for(j in 1:r){
	        	Psi[1:(b[j]+1),,j] = matrix(runif((b[j]+1)*k),b[j]+1,k) 
	          	for(c in 1:k) Psi[,c,j] = Psi[,c,j]/sum(Psi[,c,j])
	        }
	    }else upPsi = F
# parameters on initial probabilities
       	be = c(rnorm(1),rep(0,nc1))
       	if(k>2) for(h in 2:(k-1)) be = c(be,rnorm(1),rep(0,nc1))
       	out = prob_multilogit(XXdis,be,Xlab,fort)
       	Piv = out$P; Pivdis = out$Pdis
# parameters on transition probabilities
        if(param=="multilogit"){
    		Ga = matrix(rnorm((nc2+1)*(k-1),k),(nc2+1)*(k-1),k)/2
	    	PIdis = array(0,c(Zndis,k,k)); PI = array(0,c(k,k,ns,TT))
		    for(h in 1:k){
			    out = prob_multilogit(ZZdis[,,,h],Ga[,h],Zlab,fort)
			    PIdis[,,h] = out$Pdis; PI[h,,,2:TT] = array(as.vector(t(out$P)),c(1,k,ns,TT-1))
		   }
    	}else if(param=="difflogit"){
            Ga = c(-abs(rnorm(k*(k-1))),rep(0,(k-1)*nc2))
            PI = array(0,c(k,k,ns,TT))
            out = prob_multilogit(ZZdis,Ga,Zlab,fort)
   		    PIdis = out$Pdis; PI[,,,2:TT] = array(as.vector(t(out$P)),c(1,k,ns,TT-1))
		}
	}
# initialization as input
    if(start==2){
    	upPsi = T
# parameters on initial probabilities
       	be = as.vector(Be)
       	out = prob_multilogit(XXdis,be,Xlab,fort)
       	Piv = out$P; Pivdis = out$Pdis
# parameters on transition probabilities
        if(param=="multilogit"){
    		Ga = matrix(Ga,(nc2+1)*(k-1),k)
	    	PIdis = array(0,c(Zndis,k,k)); PI = array(0,c(k,k,ns,TT))
		    for(h in 1:k){
			    out = prob_multilogit(ZZdis[,,,h],Ga[,h],Zlab,fort)
			    PIdis[,,h] = out$Pdis; PI[h,,,2:TT] = array(as.vector(t(out$P)),c(1,k,ns,TT-1))
		    }
		}else if(param=="difflogit"){
            Ga = c(as.vector(t(Ga[[1]])),as.vector(Ga[[2]]))
            PI = array(0,c(k,k,ns,TT))
            out = prob_multilogit(ZZdis,Ga,Zlab,fort)
   		    PIdis = out$Pdis; PI[,,,2:TT] = array(as.vector(t(out$P)),c(1,k,ns,TT-1))
		}
    }
# dispaly output
  	cat("starting values = ",start,"\n")
  	cat("n.latent states = ",k,"\n")
  	cat("tolerance level = ",tol,"\n")
	cat(" |-------------|-------------|-------------|-------------|\n");
	cat(" |     step    |     lk      |    lk-lko   | discrepancy |\n");
	cat(" |-------------|-------------|-------------|-------------|\n");

###### standard EM #####
   	out = lk_comp_cov(S,yv,Piv,PI,Psi,k,fort=fort)    	
   	lk = out$lk; Phi = out$Phi; L = out$L; pv = out$pv
   	if(is.nan(lk)) browser()
	it = 0; lko = lk-10^10; lkv = NULL
	par = c(as.vector(Piv),as.vector(PI),as.vector(Psi)); paro = par
# Iterate until convergence
t0 = proc.time()
   	cat("",sprintf("%11g",c(0,lk)),"\n",sep=" | ")
	while(((abs(lk-lko)>tol | max(abs(par-paro))>tol) & it<10^3) || it<2){
		Psi0 = Psi; Piv0 = Piv; PI0 = PI
		it = it+1
# ---- E-step ----
# Compute V and U
       	out = prob_post_cov(S,yv,Psi,Piv,PI,Phi,L,pv,fort=fort)
       	U = out$U; V = out$V	
# If required store parameters
# ---- M-step ----
# Update Psi
       	if(upPsi){
    		Y1 = array(0,c(mb+1,k,r))
	    	Vv = matrix(aperm(V,c(1,3,2)),ns*TT,k)
		    for(j in 1:r) for(jb in 0:b[j]) {
	    		ind = which(Sv[,j]==jb)
	    		Y1[jb+1,,j] = colSums(Vv[ind,])				
	    	}
	    	for(c in 1:k) for(j in 1:r) Psi[,c,j] = Y1[,c,j]/sum(Y1[,c,j])
		}
# Update piv
		out = est_multilogit(V[,,1],XXdis,Xlab,be,Pivdis,fort=fort)
		be = out$be; Pivdis = out$Pdi; Piv = out$P
# Update Pi
		if(param=="multilogit"){
	    	for(h in 1:k){
		    	UU = NULL
		    	for(t in 2:TT) UU = rbind(UU,t(U[h,,,t]))
		    	out = est_multilogit(UU,ZZdis[,,,h],Zlab,Ga[,h],PIdis[,,h],fort=fort)
		    	PIdis[,,h] = out$Pdis; PI[h,,,2:TT] = t(out$P); Ga[,h] = out$be
	   	 	}
		}else if(param=="difflogit"){
		    Tmp = aperm(U[,,,2:TT],c(1,3,4,2))
		    Tmp = matrix(Tmp,ns*k*(TT-1),k)
           	out = est_multilogit(Tmp,ZZdis,Zlab,Ga,PIdis,fort=fort)
	    	PIdis = out$Pdis; Ga = out$be
	    	Tmp = array(out$P,c(k,ns,TT-1,k))
	    	PI[,,,2:TT] = aperm(Tmp,c(1,4,2,3)) 
        }
# Compute log-likelihood
   		paro = par; par = c(as.vector(Piv),as.vector(PI),as.vector(Psi));
   		lko = lk;
   		out = lk_comp_cov(S,yv,Piv,PI,Psi,k,fort=fort)
   		lk = out$lk; Phi = out$Phi; L = out$L; pv = out$pv
# Display output
       	if(it/10 == floor(it/10)){
       		cat("",sprintf("%11g",c(it,lk,lk-lko,max(abs(par-paro)))),"\n",sep=" | ")
       	}
   		lkv = c(lkv,lk)
	}
	if(it/10 > floor(it/10)) cat("",sprintf("%11g",c(it,lk,lk-lko,max(abs(par-paro)))),"\n",sep=" | ")

#### compute infomation matrix ####
	if(info){
   	    dlPsi = array(0,c(mb+1,k,r,k*sb))
   	    count = 0
	    for(c in 1:k) for(j in 1:r){
	    	ind = count+(1:b[j])
	        temp = pmax(Psi[,c,j],10^-50)
	    	dlPsi[,c,j,ind] = (diag(b[j]+1)-rep(1,b[j]+1)%o%temp)%*%Am[[j]]
	    	count = count+b[j]
	    	th = c(th,log(temp[-1]/temp[1]))
		}
		dlPiv = array(0,c(ns,k,(1+nc1)*(k-1)))
		for(j in 1:Xndis){
	        temp = pmax(Pivdis[j,],10^-50)
	        Temp = (diag(k)-rep(1,k)%o%temp)%*%XXdis[,,j]
	    	for(i in which(Xlab==j)) dlPiv[i,,] = Temp
        }
        th = c(th,be)
        count = 0
        dlPI = array(0,c(k,k,ns*TT,(1+nc2)*(k-1)*k))
        if(param=="multilogit"){
        	temp0 = rep(1,k); Temp0 = diag(k)
		    for(h in 1:k){
		    	ind = count+(1:((1+nc2)*(k-1)))
		    	for(j in 1:Zndis){
		    		temp = pmax(PIdis[j,,h],10^-50)
           	        Temp = (Temp0-temp0%o%temp)%*%ZZdis[,,j,h]
           	        for(i in which(Zlab==j)) dlPI[h,,ns+i,ind] = Temp
		    	}
		    	count = count+(1+nc2)
                th = c(th,Ga[,h])		    
		    }
            dlPI = array(dlPI,c(k,k,ns,TT,(1+nc2)*(k-1)*k))
		}else if(param=="difflogit"){
            Ga = c(as.vector(t(Ga[[1]])),as.vector(Ga[[2]]))
            PI = array(0,c(k,k,ns,TT))
            out = prob_multilogit(ZZdis,Ga,Zlab,fort)
   		    PIdis = out$Pdis; PI[,,,2:TT] = array(as.vector(t(out$P)),c(1,k,ns,TT-1))
		}
# Compute log-likelihood
    	out = lk_comp_cov(S,yv,Piv,PI,Psi,k,der=TRUE,fort=fort,dlPsi=dlPsi,dlPiv=dlPiv,dlPI=dlPI)
    	sc = out$dlk; dlL = out$dlL; dlPhi = out$dlPhi; dlL2 = out$dlL2; dlpv = out$dlpv
    	lk = out$lk; Phi = out$Phi; L = out$L; pv = out$pv
	    if(is.nan(lk)) browser()
  		it = 0; lko = lk-10^10; lkv = NULL; dev = NULL
# backward recursion
        out = prob_post_cov(S,yv,Psi,Piv,PI,Phi,L,pv,der=TRUE,fort=fort,dlPhi=dlPhi,dlPiv,
   	                        dlPI=dlPI,dlL=dlL,dlL2=dlL2,dlpv=dlpv)
       	U = out$U; V = out$V; dlU = out$dlU; dlV = out$dlV
# ---- M-step ----
# score and info Psi
       	sc = NULL
   		Y1 = array(0,c(mb+1,k,r))
    	Vv = matrix(aperm(V,c(1,3,2)),ns*TT,k)
	    for(j in 1:r) for(jb in 0:b[j]) {
	    	ind = which(Sv[,j]==jb)
	    	Y1[jb+1,,j] = colSums(Vv[ind,])				
	    }
	    for(c in 1:k) for(j in 1:r){
	    	sc = c(sc,t(Am[[j]])%*%(Y1[,c,j]-sum(Y1[,c,j])*Psi[,c,j]))
	        Psi[,c,j] = Y1[,c,j]/sum(Y1[,c,j])
   	        temp = pmax(Psi[,c,j],10^-50)
            Op = diag(temp)-temp%o%temp
	        Temp  = sum(Y1[,c,j])*t(Am[[j]])%*%Op%*%Am[[j]]
            if(j==1 & c==1) Fi = Temp else Fi = bdiag(Fi,Temp)
	    }
# score and info piv
		out = est_multilogit(V[,,1],XXdis,Xlab,be,Pivdis,fort=fort,ex=TRUE)
		sc = c(sc,out$sc); Fi = bdiag(Fi,out$Fi)
# score and info Pi
        if(param=="multilogit"){
		    for(h in 1:k){
			    UU = NULL
			    for(t in 2:TT) UU = rbind(UU,t(U[h,,,t]))
		    	out = est_multilogit(UU,ZZdis[,,,h],Zlab,Ga[,h],PIdis[,,h],fort=fort,ex=TRUE)
			    sc = c(sc,out$sc); Fi = bdiag(Fi,out$Fi) 
	    	}
		}else if(param=="difflogit"){
	    	Tmp = aperm(U[,,,2:TT],c(1,3,4,2))
	    	Tmp = matrix(Tmp,ns*k*(TT-1),k)
           	out = est_multilogit(Tmp,ZZdis,Zlab,Ga,PIdis,fort=fort)
	    	PIdis = out$Pdis; Ga = out$be
	    	Tmp = array(out$P,c(k,ns,TT-1,k))
	    	PI[,,,2:TT] = aperm(Tmp,c(1,4,2,3)) 
        	Fi = bdiag(Fi,out$Fi)
       	}
       	Fi = as.matrix(Fi)
# compute correction matrix for the information
   		nal = dim(dlPhi)[4]; nbe = dim(dlPiv)[3]; nga = dim(dlPI)[5]
		npar = nal+nbe+nga
		Cor = matrix(0,npar,npar)
		dY1 = array(0,c(mb+1,k,r,npar))
		dV = array(V,c(ns,k,TT,npar))*dlV
    	dVv = array(aperm(dV,c(1,3,2,4)),c(ns*TT,k,nal+nbe+nga))
		for(j in 1:r) for(jb in 0:b[j]) {
			ind = which(Sv[,j]==jb)
			for(h in 1:(nal+nbe+nga)) dY1[jb+1,,j,h] = colSums(dVv[ind,,h])
		}
		for(h in 1:(npar)){
			count = 0
			for(c in 1:k) for(j in 1:r){
				count = count+1
				ind = (count-1)*mb+(1:mb)
				Cor[h,ind] = t(Am[[j]])%*%(dY1[,c,j,h]-sum(dY1[,c,j,h])*Psi[,c,j])
			}
		}
		for(h in 1:(npar)){
			out = est_multilogit(dV[,,1,h],XXdis,Xlab,be,Pivdis,fort=fort,ex=TRUE)
			Cor[h,nal+(1:nbe)] = out$sc
		}
		dU = array(U,c(k,k,ns,TT,npar))*dlU
        if(param=="multilogit"){
   	    	rGa = dim(Ga)[1]
		    for(h in 1:k){
	    		for(h1 in 1:npar){
				    UU = NULL
			    	for(t in 2:TT) UU = rbind(UU,t(dU[h,,,t,h1]))
			    	out = est_multilogit(UU,ZZdis[,,,h],Zlab,Ga[,h],PIdis[,,h],fort=fort,ex=TRUE)
			    	ind = nal+nbe+(h-1)*rGa+(1:rGa)
			    	Cor[h1,ind] = out$sc
				}
		    }
		}else if(param=="difflogit"){
	    	Tmp = aperm(U[,,,2:TT],c(1,3,4,2))
		    Tmp = matrix(Tmp,ns*k*(TT-1),k)
   	        out = est_multilogit(Tmp,ZZdis,Zlab,Ga,PIdis,fort=fort)
		    PIdis = out$Pdis; Ga = out$be
	    	Tmp = array(out$P,c(k,ns,TT-1,k))
		    PI[,,,2:TT] = aperm(Tmp,c(1,4,2,3)) 
   	    }
# check score and information
		if(check_der){
	        lk0 = lk
			out = lk_obs_cov(th,S,b,yv,Am,XXdis,Xlab,ZZdis,Zlab,param,fort)
			sc1 = out$sc
			print(lk-lk0)
       		lth = length(th)
        	scn = rep(0,lth); Fn = matrix(0,lth,lth)
   	    	for(h in 1:lth){
   		    	thh = th; thh[h] = thh[h]+10^-6
           		outh = lk_obs_cov(thh,S,b,yv,Am,XXdis,Xlab,ZZdis,Zlab,param,fort=fort)
           		scn[h] = (outh$lk-lk)*10^6
           		Fn[,h] = -(outh$sc-sc)*10^6
   	 		}
	     	print(lk-lk0)
   		    print(round(cbind(sc,sc1,scn,sc-scn),4))
   		    print(round(cbind(diag(Fi-Cor),diag(Fn),diag(Fi-Cor-Fn)),4))
		}
# Information matrix and standard errors
    	Fi = Fi-Cor
		iFi = ginv(Fi)		
		se = sqrt(diag(iFi))
# Divide parameters
      	sepsi = se[1:nal]
        sebe = se[nal+(1:nbe)]
        sega = se[nal+nbe+(1:nga)]
	}	
# Compute number of parameters
    if(r==1) np = k*mb*r else np = k*sum(b)
    np = np+(k-1)*(nc1+1)
    if(param=="multilogit") np = np+(k-1)*(nc2+1)*k else if(param=="difflogit")  np = np+(k-1)*(nc2+k)
  	aic = -2*lk+np*2
  	bic = -2*lk+np*log(n)
#	out = list(lk=lk,piv=piv,Pi=Pi,Psi=Psi,np=np,aic=aic,bic=bic,lkv=lkv,J=J,V=V1,th=th,sc=sc)
# local decoding
    Ul = matrix(0,ns,TT)
    for(i in 1:ns) for(t in 1:TT){
    	Ul[i,t] = which.max(V[i,,t])
    }
    if(all(yv==1)) V1=V
# final output
    if(output==F){
    	V1 = NULL
    	PI = NULL
    	Piv = NULL
    	Ul = NULL
    }
    if(info){
    	psi = as.vector(aperm(Psi,c(1,3,2)))
    	dPsi = diag(psi)%*%matrix(aperm(dlPsi,c(1,3,2,4)),c((b+1)*k,nal)) 
    	sePsi = sqrt(diag(dPsi%*%iFi[1:nal,1:nal]%*%t(dPsi)))
    	sePsi = aperm(array(sePsi,c(b+1,r,c)),c(1,3,2))
    }else{
    	sePsi = NULL
    }
	Be = matrix(be,nc1+1,k-1)
	if(info>0) seBe = matrix(sebe,nc1+1,k-1) else seBe = NULL
	if(param=="multilogit"){
		if(k>2) Ga = array(as.vector(Ga),c(nc2+1,k-1,k))
		if(info>0){
			if(k==2){
				seGa = matrix(sega,nc2+1,2)
			}else if(k>2){
				seGa = array(as.vector(sega),c(nc2+1,k-1,k))
			}
		}else{
			seGa = NULL
		}
	}else if(param=="difflogit"){
		Ga0 = Ga
		Ga = vector("list",2)
		Ga[[1]] = t(matrix(Ga0[1:(k*(k-1))],k-1,k))
		Ga[[2]] = matrix(Ga0[(k*(k-1))+(1:((k-1)*nc2))],nc2,k-1)
	}
	out = list(lk=lk,Be=Be,Piv=Piv,Ga=Ga,PI=PI,Psi=Psi,np=np,aic=aic,bic=bic,lkv=lkv,V=V1,Ul=Ul,
	           sePsi=sePsi,seBe=seBe,seGa=seGa)
    cat(" |-------------|-------------|-------------|-------------|\n");
	return(out)
}
#-------------------------------------------------  
lk_comp_cov <- function(S,yv,Piv,PI,Psi,k,fort=FALSE,der=FALSE,
                       dlPsi=NULL,dlPiv=NULL,dlPI=NULL){

# der = TRUE for derivative

# Preliminaries
   	sS = dim(S)
  	ns = sS[1]
  	TT = sS[2]
  	l = max(S)+1
  	if(length(sS)==2) r = 1 else r = sS[3]
# Compute log-likelihood
	Phi = array(1,c(ns,k,TT)); L = array(0,c(ns,k,TT))
	if(fort){
        if(r==1){
	       for(t in 1:TT) Phi[,,t] = Phi[,,t]*Psi[S[,t]+1,,1]
	    }else{
            o = .Fortran("for_mult",as.integer(TT),as.integer(r),as.integer(k),as.integer(ns),as.integer(l),as.integer(S),Psi,Piv,PI,Phi=Phi,LL=array(0,c(ns,k,TT)))
            Phi = o$Phi; L = o$LL
	    }
	}else{
		if(r==1){
	        for(t in 1:TT) Phi[,,t] = Phi[,,t]*Psi[S[,t]+1,,1]
	    }else{
	        for(t in 1:TT) for(j in 1:r) Phi[,,t] = Phi[,,t]*Psi[S[,t,j]+1,,j]
	    }
	}  	
    if(r==1 || fort==F){
      	L[,,1] = Phi[,,1]*Piv
		for(t in 2:TT){	
  			for(i in 1:ns)	L[i,,t] = L[i,,t-1]%*%PI[,,i,t]
  			L[,,t] = L[,,t]*Phi[,,t]
  		}
  	}
  	pv = rowSums(L[,,TT])
  	lk = sum(yv*log(pv))
# if the derivative is required
  	if(der){
  		nal = dim(dlPsi)[4]; nbe = dim(dlPiv)[3]; nga = dim(dlPI)[5]
  		npar = nal+nbe+nga
  		indal = 1:nal; indbe = nal+(1:nbe); indga = nal+nbe+(1:nga)
        dlPhi = array(0,c(ns,k,TT,nal)); dlL = array(0,c(ns,k,TT,nal+nbe+nga))
		if(r==1){
	        for(t in 1:TT) dlPhi[,,t,] = dlPsi[S[,t]+1,,1,]
	    }else{
	        for(t in 1:TT) for(j in 1:r) dlPhi[,,t,] = dlPhi[,,t,]+dlPsi[S[,t,j]+1,,j,]
	    }
	    dlL[,,1,indal] = dlPhi[,,1,]; dlL[,,1,indbe] = dlPiv
		dlL2 = array(0,c(ns,k,k,t,npar))
		for(t in 2:TT){
			L2 = array(0,c(ns,k,k))
			for(ub in 1:k) for(u in 1:k){
   				L2[,ub,u] = L[,ub,t-1]*PI[ub,u,,t]*Phi[,u,t]
   				dlL2[,ub,u,t,] = dlL[,ub,t-1,]
   				dlL2[,ub,u,t,indal] = dlL2[,ub,u,t,indal]+dlPhi[,u,t,]
   				dlL2[,ub,u,t,indga] = dlL2[,ub,u,t,indga]+dlPI[ub,u,,t,]
  			}
            for(ub in 1:k) for(u in 1:k) dlL[,u,t,] = dlL[,u,t,]+L2[,ub,u]/L[,u,t]*dlL2[,ub,u,t,]
        }
        dlpv = matrix(0,ns,npar)
        for(u in 1:k) dlpv = dlpv+L[,u,TT]/pv*dlL[,u,TT,]
        dlk = colSums(dlpv)
  	}else{
  		dlk=NULL; dlL=NULL; dlL2=NULL; dlpv = NULL; dlPhi=NULL
  	}
# output
  	out = list(lk=lk,Phi=Phi,L=L,pv=pv,dlk=dlk,dlPhi=dlPhi,dlL=dlL,dlL2=dlL2,dlpv=dlpv)
}  
#------------------------------------------------------
invglob <- function(eta){

	b = length(eta)
	if(b == 1) p = c(1,exp(eta))/(1+exp(eta))
	else{
		pc = c(1/(1+exp(eta)),1)
	  	Di = cbind(rep(0,b),diag(b))-cbind(diag(b),rep(0,b))
	  	Di = rbind(c(1,rep(0,b)), Di)
	  	p = Di%*%pc
	}
}
#-----------------------------------------------------------------------------------------------------
est_lm_cov_2step <- function(S,X,k,start=0,tol=10^-6,param="multilogit",fort=fort,indcov=NULL,indsel=NULL,flag=F){

# flag: F for direct assignment to the latent class, T = two step estimator until convergence

# fist LC model with random starting values and assign subjects to classes
    n = dim(S)[1]
    TT = dim(S)[2]
    r = dim(S)[3]
    S2 = matrix(S,n*TT,r)
    out = aggr_data(S2)
    S2 = out$data_dis
    freq2 = out$freq
    label = out$label
    out = lcest_poly_miss_fort(S2,yv=freq2,k)
    for(j in 1:(10*k)){
        outj = lcest_poly_miss_fort(S2,yv=freq2,k)
    	    if(outj$lk>out$lk) out=outj
    }
    ind = order(t(out$La[,,1])%*%(0:max(S)))
    Psi = out$La[,ind,]
    Pp = out$Pp[,ind]
    u = apply(Pp,1,which.max)
    v = u[label]
    V = matrix(v,n,TT)
# fit LM model
    Psi0 = array(diag(k),c(k,k,1))
    if(flag){
      out2 = est_lm_cov(S,X,rep(1,n),k,tol=tol,param=param,Psi=Psi,fort=fort)    	
    }else{
      out2 = est_lm_cov(V-1,X,rep(1,n),k,tol=tol,param=param,Psi=Psi0,fort=fort)
    }
# fit separate LM models for subgroups
    if(is.null(indcov) || is.null(indsel)){
    	out_groups = NULL
    }else{
    	ng = max(indsel)
    	out_groups = vector("list",ng)
    	for(g in 1:ng){
    		print(g)
    		ind = (indsel==g)
    		if(flag){
    		  out_groups[[g]] = est_lm_cov(S[ind,,],X[ind,,indcov],rep(1,sum(ind)),k,tol=tol,param=param,Psi=Psi,fort=fort)
    		}else{
    		  out_groups[[g]] = est_lm_cov(V[ind,]-1,X[ind,,indcov],rep(1,sum(ind)),k,tol=tol,param=param,Psi=Psi0,fort=fort)
    		}
    	}
    }
# output
    out = list(lk=out2$lk,Be=out2$Be,Piv=out2$Piv,Ga=out2$Ga,PI=out2$PI,Psi=out2$Psi,
    np=out2$np,aic=out2$aic,bic=out2$bic,lkv=out2$lkv,V=out2$V1,Ul=out2$Ul,piv0=out$piv,Psi0=Psi,lk0=out$lk,out_groups=out_groups)
    
}


